@@ -133,12 +133,30 @@ public class AltosIdleFetch implements AltosDataProvider {
 			       AltosIdler.idle_ms5607,
 			       AltosIdler.idle_sensor_metrum),
 
-		new AltosIdler("TeleMega",
+		new AltosIdler("TeleMega-v0",
 			       AltosIdler.idle_gps,
 			       AltosIdler.idle_mma655x,
 			       AltosIdler.idle_ms5607,
 			       AltosIdler.idle_imu, AltosIdler.idle_mag,
 			       AltosIdler.idle_sensor_mega),
+		new AltosIdler("TeleMega-v1",
+			       AltosIdler.idle_gps,
+			       AltosIdler.idle_mma655x,
+			       AltosIdler.idle_ms5607,
+			       AltosIdler.idle_imu, AltosIdler.idle_mag,
+			       AltosIdler.idle_sensor_mega),
+		new AltosIdler("TeleMega-v2",
+			       AltosIdler.idle_gps,
+			       AltosIdler.idle_mma655x,
+			       AltosIdler.idle_ms5607,
+			       AltosIdler.idle_imu, AltosIdler.idle_mag,
+			       AltosIdler.idle_sensor_mega),
+		new AltosIdler("TeleMega-v3",
+			       AltosIdler.idle_gps,
+			       AltosIdler.idle_mma655x,
+			       AltosIdler.idle_ms5607,
+			       AltosIdler.idle_imu,
+			       AltosIdler.idle_sensor_mega),
 		new AltosIdler("EasyMega",
 			       AltosIdler.idle_mma655x,
 			       AltosIdler.idle_ms5607,
