@@ -10,9 +10,13 @@
 
 #ifndef REGTEST
 
+#undef ao_getchar
+
+extern int ao_getchar(void);
+
 int fgetc( struct _PDCLIB_file_t * stream )
 {
-    return getchar();
+    return ao_getchar();
 }
 
 #endif
