@@ -223,11 +223,12 @@ main(void)
 	ao_cmd_init();
 //	ao_lcd_stm_init();
 //	ao_lcd_font_init();
-//	ao_spi_init();
+	ao_spi_init();
 //	ao_i2c_init();
 //	ao_exti_init();
 //	ao_quadrature_init();
 //	ao_button_init();
+	ao_storage_init();
 
 //	ao_timer_set_adc_interval(100);
 
