@@ -26,8 +26,6 @@
 #define AO_PLLMUL		12
 #define AO_RCC_CFGR_PLLMUL	(STM_RCC_CFGR_PLLMUL_12)
 
-#define AO_CC1200_FOSC		40000000
-
 /* SYSCLK = 32MHz (no need to go faster than CPU) */
 #define AO_PLLDIV		3
 #define AO_RCC_CFGR_PLLDIV	(STM_RCC_CFGR_PLLDIV_3)
@@ -79,23 +77,28 @@
 #define FAST_TIMER_FREQ		10000	/* .1ms for debouncing */
 
 /*
- * Radio is a cc1200 connected via SPI
+ * Radio is a cc1120 connected via SPI
  */
 
-#define AO_RADIO_CAL_DEFAULT 	5695733
+#define AO_RADIO_CAL_DEFAULT 	0x6ca333
 
 #define AO_FEC_DEBUG		0
-#define AO_CC1200_SPI_CS_PORT	(&stm_gpiod)
-#define AO_CC1200_SPI_CS_PIN	0
-#define AO_CC1200_SPI_BUS	AO_SPI_2_PD1_PD3_PD4
-#define AO_CC1200_SPI		stm_spi2
-#define AO_CC1200_SPI_SPEED	AO_SPI_SPEED_FAST
+#define AO_CC1120_SPI_CS_PORT	(&stm_gpiod)
+#define AO_CC1120_SPI_CS_PIN	0
+#define AO_CC1120_SPI_BUS	AO_SPI_2_PD1_PD3_PD4
+#define AO_CC1120_SPI		stm_spi2
 
-#define AO_CC1200_INT_PORT		(&stm_gpioc)
-#define AO_CC1200_INT_PIN		(15)
+#define AO_CC1120_INT_PORT		(&stm_gpioc)
+#define AO_CC1120_INT_PIN		(15)
+#define AO_CC1120_MCU_WAKEUP_PORT	(&stm_gpiod)
+#define AO_CC1120_MCU_WAKEUP_PIN	(2)
+
+#define AO_CC1120_INT_GPIO	2
+#define AO_CC1120_INT_GPIO_IOCFG	CC1120_IOCFG2
+
+#define AO_CC1120_MARC_GPIO	3
+#define AO_CC1120_MARC_GPIO_IOCFG	CC1120_IOCFG3
 
-#define AO_CC1200_INT_GPIO	2
-#define AO_CC1200_INT_GPIO_IOCFG	CC1200_IOCFG2
 
 #define LOW_LEVEL_DEBUG		0
 
@@ -250,6 +253,7 @@
 #define AO_QUADRATURE_1_B	0
 
 #define AO_QUADRATURE_BOX	1
+#define AO_QUADRATURE_DEBOUNCE	10
 
 /*
  * Buttons
